package com.ruoyi.advertiser25.controller;

import com.ruoyi.advertiser25.domain.AdHandle25;
import com.ruoyi.advertiser25.domain.Material25;
import com.ruoyi.advertiser25.domain.PlanSetting25;
import com.ruoyi.advertiser25.enums.MaterialStatusEnum25;
import com.ruoyi.advertiser25.service.IPlanSettingService;
import com.ruoyi.advertiser25.service.IScreenDeviceService;
import com.ruoyi.advertiser25.service.IScreenMaterialService;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import org.apache.commons.collections4.Get;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 把手 Controller
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@RestController
@RequestMapping("/advertising25/setting")
public class PlanSettingController extends BaseController {

    @Resource
    private IPlanSettingService planSettingService;

    @Resource
    private IScreenDeviceService screenDeviceService;

    @Resource
    private IScreenMaterialService screenMaterialService;

    @GetMapping("/deviceDict")
    public AjaxResult deviceDict(Long groupId){
        return AjaxResult.success(screenDeviceService.selectAdHandleList(new AdHandle25().setGroupId(groupId)));
    }

    @GetMapping("/materialDict")
    public AjaxResult materialDict(){
        return AjaxResult.success(screenMaterialService.selectMaterialList(new Material25().setPicShow(true).setStatus(MaterialStatusEnum25.PASS.getCode())));
    }

    @GetMapping("/getMaterialById")
    public AjaxResult getMaterialById(Long id){
        return AjaxResult.success(screenMaterialService.selectMaterialById(id));
    }
    /**
     * 查询分组列表
     */
    @GetMapping("/list")
    public TableDataInfo list(PlanSetting25 planSetting) {
        startPage();
        List<PlanSetting25> list = planSettingService.selectPlanSettingList(planSetting);
        return getDataTable(list);
    }

    /**
     * 获取详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(planSettingService.selectPlanSettingById(id));
    }

    /**
     * 新增
     */
    @PostMapping
    public AjaxResult add(@RequestBody PlanSetting25 planSetting25) {
        if (UserConstants.NOT_UNIQUE.equals(planSettingService.checkNumUnique(planSetting25))) {
            return AjaxResult.error("新增分组'" + planSetting25.getGroupName() + "'失败，分组名称已存在");
        }
        return toAjax(planSettingService.insertPlanSetting(planSetting25));
    }

    /**
     * 修改
     */
    @PutMapping
    public AjaxResult edit(@RequestBody PlanSetting25 planSetting25) {
        if (UserConstants.NOT_UNIQUE.equals(planSettingService.checkNumUnique(planSetting25))) {
            return AjaxResult.error("修改分组'" + planSetting25.getGroupName() + "'失败，分组名称已存在");
        }
        return toAjax(planSettingService.updatePlanSetting(planSetting25));
    }

    /**
     * 删除
     */
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(planSettingService.deletePlanSettingByIds(ids));
    }

}
