package com.ruoyi.advertiser25.controller;

import com.ruoyi.advertiser25.domain.AdHandle25;
import com.ruoyi.advertiser25.service.IScreenDeviceService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 把手 Controller
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@RestController
@RequestMapping("/advertising25/handle")
public class ScreenDeviceController extends BaseController {

    @Resource
    private IScreenDeviceService adHandleService;

    /**
     * 查询把手 列表
     */
    @GetMapping("/list")
    public TableDataInfo list(AdHandle25 adHandle25)
    {
        startPage();
        List<AdHandle25> list = adHandleService.selectAdHandleList(adHandle25);
        return getDataTable(list);
    }

    /**
     * 获取把手 详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(adHandleService.selectAdHandleById(id));
    }

    /**
     * 新增把手
     */
    @Log(title = "把手 ", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AdHandle25 adHandle25)
    {
        if (UserConstants.NOT_UNIQUE.equals(adHandleService.checkNumUnique(adHandle25))) {
            return AjaxResult.error("新增把手'" + adHandle25.getNum() + "'失败，编号已存在");
        }
        return toAjax(adHandleService.insertAdHandle(adHandle25));
    }

    /**
     * 修改把手
     */
    @Log(title = "把手 ", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AdHandle25 adHandle25)
    {
        if (UserConstants.NOT_UNIQUE.equals(adHandleService.checkNumUnique(adHandle25))) {
            return AjaxResult.error("修改把手'" + adHandle25.getNum() + "'失败，编号已存在");
        }
        return toAjax(adHandleService.updateAdHandle(adHandle25));
    }

    /**
     * 删除把手
     */
    @Log(title = "把手 ", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(adHandleService.deleteAdHandleByIds(ids));
    }

}
