package com.ruoyi.advertiser25.controller;

import com.ruoyi.advertiser25.domain.Upgrade25;
import com.ruoyi.advertiser25.service.IUpgradeService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

/**
 * 固件升级 Controller
 * 
 * @author caiweigan
 * @date 2022-02-07
 */
@RestController
@RequestMapping("/advertising25/upgrade")
public class UpgradeController extends BaseController {

    @Resource
    private IUpgradeService upgradeService;

    /**
     * 查询固件升级 列表
     */
    @GetMapping("/list")
    public TableDataInfo list(Upgrade25 adFirmwareUpgrade)
    {
        startPage();
        List<Upgrade25> list = upgradeService.selectAdFirmwareUpgradeList(adFirmwareUpgrade);
        return getDataTable(list);
    }

    /**
     * 导出固件升级 列表
     */
    @GetMapping("/export")
    public AjaxResult export(Upgrade25 adFirmwareUpgrade)
    {
        List<Upgrade25> list = upgradeService.selectAdFirmwareUpgradeList(adFirmwareUpgrade);
        ExcelUtil<Upgrade25> util = new ExcelUtil<>(Upgrade25.class);
        return util.exportExcel(list, "固件升级 数据");
    }

    /**
     * 获取固件升级 详细信息
     */
    @GetMapping(value = "/{upgradeId}")
    public AjaxResult getInfo(@PathVariable("upgradeId") Long upgradeId)
    {
        return AjaxResult.success(upgradeService.selectAdFirmwareUpgradeByUpgradeId(upgradeId));
    }

    /**
     * 新增固件升级 
     */
    @PostMapping
    public AjaxResult add(@RequestBody Upgrade25 adFirmwareUpgrade)
    {
        return toAjax(upgradeService.insertAdFirmwareUpgrade(adFirmwareUpgrade));
    }

    /**
     * 修改固件升级 
     */
    @PutMapping
    public AjaxResult edit(@RequestBody Upgrade25 adFirmwareUpgrade)
    {
        return toAjax(upgradeService.updateAdFirmwareUpgrade(adFirmwareUpgrade));
    }

    /**
     * 删除固件升级 
     */
	@DeleteMapping("/{upgradeIds}")
    public AjaxResult remove(@PathVariable Long[] upgradeIds)
    {
        return toAjax(upgradeService.deleteAdFirmwareUpgradeByUpgradeIds(upgradeIds));
    }

    /**
     * 发布
     */
    @PutMapping("/publish_status/{upgradeId}")
    public AjaxResult publish(@PathVariable Long upgradeId) {
        return toAjax(upgradeService.updatePublishStatus(upgradeId));
    }

    /**
     * 上传
     */
    @PostMapping("/upload")
    public AjaxResult upload(MultipartFile file) throws IOException {
        Upgrade25 upgrade = upgradeService.upload(file);
        return AjaxResult.success(upgrade);
    }
}
