package com.ruoyi.advertiser25.mapper;

import com.ruoyi.advertiser25.domain.AdHandle25;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 把手 Mapper接口
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
public interface ScreenDeviceMapper
{
    /**
     * 查询把手
     *
     * @param id 把手 ID
     * @return 把手
     */
    AdHandle25 selectAdHandleById(Long id);

    /**
     * 查询把手 列表
     *
     * @param adHandle25 把手
     * @return 把手 集合
     */
    List<AdHandle25> selectAdHandleList(AdHandle25 adHandle25);

    /**
     * 新增把手
     *
     * @param adHandle25 把手
     * @return 结果
     */
    int insertAdHandle(AdHandle25 adHandle25);

    /**
     * 修改把手
     *
     * @param adHandle25 把手
     * @return 结果
     */
    int updateAdHandle(AdHandle25 adHandle25);

    /**
     * 删除把手
     *
     * @param id 把手 ID
     * @return 结果
     */
    int deleteAdHandleById(Long id);

    /**
     * 批量删除把手
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    int deleteAdHandleByIds(Long[] ids);

    /**
     * 根据编号查询把手
     * @param num
     * @return
     */
    AdHandle25 selectAdHandleByNum(String num);

    /**
     * 根据imei查询把手
     */
    AdHandle25 selectAdHandleByImei(String imei);

    /**
     * 验证把手编号是否重复
     * @param adHandle25
     * @return
     */
    int checkNumUnique(AdHandle25 adHandle25);

    /**
     * 更新关联关系
     */
	int updateAdHandleRelation(AdHandle25 adHandle25);

    /**
     * 根据心跳修改拉手信息
     */
    void updateHandleByHeartbeat(AdHandle25 adHandle25);

    /**
     * 定时器修改拉手状态
     */
    int updateOnlineStatusTask(@Param("params") Map<String,Object> params);

    /**
     * 修改seId
     */
    void updateSeId(@Param("imei") String imei, @Param("seId") String seId);

    /**
     * 通过imei取消拉手绑定信息
     */
    void unbindHandleByImei(@Param("imei") String imei);

    void unbindHandleByGroupId(Long groupId);

    void bindHandleByImei(@Param("imeiList") List<String> imeiList, @Param("groupId") Long groupId, @Param("groupName") String groupName);
}
