package com.ruoyi.advertiser25.service.impl;

import cn.hutool.core.convert.Convert;
import com.ruoyi.advertiser25.domain.PlanSetting25;
import com.ruoyi.advertiser25.mapper.PlanSettingMapper;
import com.ruoyi.advertiser25.mapper.ScreenDeviceMapper;
import com.ruoyi.advertiser25.service.IPlanSettingService;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 把手 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class PlanSettingServiceImpl implements IPlanSettingService {

    @Resource
    private PlanSettingMapper planSettingMapper;

    @Resource
    private ScreenDeviceMapper screenDeviceMapper;

    @Override
    public PlanSetting25 selectPlanSettingById(Long id) {
        return planSettingMapper.selectPlanSettingById(id);
    }

    @Override
    public List<PlanSetting25> selectPlanSettingList(PlanSetting25 planSetting25) {
        List<PlanSetting25> list = planSettingMapper.selectPlanSettingList(planSetting25);
        return list;
    }

    @Override
    public int insertPlanSetting(PlanSetting25 planSetting25) {
        planSetting25.setCreateBy(SecurityUtils.getUsername());
        planSetting25.setCreateTime(DateUtils.getNowDate());
        int result = planSettingMapper.insertPlanSetting(planSetting25);
        // 处理绑定的屏幕
        upDeviceBind(planSetting25.getImeis(), planSetting25.getId(), planSetting25.getGroupName());
        return result;
    }

    private void upDeviceBind(String imeis, Long groupId, String groupName){
        List<String> imeiList = Arrays.asList(imeis.split(","));
        screenDeviceMapper.unbindHandleByGroupId(groupId);
        screenDeviceMapper.bindHandleByImei(imeiList, groupId, groupName);
    }

    @Override
    public int updatePlanSetting(PlanSetting25 planSetting25) {
        planSetting25.setUpdateBy(SecurityUtils.getUsername());
        planSetting25.setUpdateTime(DateUtils.getNowDate());
        // 处理绑定的屏幕
        upDeviceBind(planSetting25.getImeis(), planSetting25.getId(), planSetting25.getGroupName());
        return planSettingMapper.updatePlanSetting(planSetting25);
    }

    @Override
    public int deletePlanSettingByIds(Long[] ids) {
        for (Long id: ids) {
            screenDeviceMapper.unbindHandleByGroupId(id);
        }
        return planSettingMapper.deletePlanSettingByIds(ids);
    }

    /**
     * 验证编号是否重复
     */
    @Override
    public String checkNumUnique(PlanSetting25 planSetting25) {
        int count = planSettingMapper.checkNumUnique(planSetting25);
        return count > 0 ? UserConstants.NOT_UNIQUE : UserConstants.UNIQUE;
    }
}
