package com.ruoyi.advertiser25.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.ruoyi.advertiser25.constant.SubwayConstant25;
import com.ruoyi.advertiser25.domain.CarouselPlan25;
import com.ruoyi.advertiser25.domain.PlanSetting25;
import com.ruoyi.advertiser25.domain.dto.*;
import com.ruoyi.advertiser25.domain.req.CarouselPlanParamReq25;
import com.ruoyi.advertiser25.mapper.PlanSettingMapper;
import com.ruoyi.advertiser25.mapper.RotationPlanMapper;
import com.ruoyi.advertiser25.mapper.ScreenMaterialMapper;
import com.ruoyi.advertiser25.service.IRotationPlanService;
import com.ruoyi.advertiser25.service.ISysParamService;
import com.ruoyi.advertiser25.task.CreateCarouselPlanUtil25;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 广告轮播计划 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-24
 */
@Service
public class RotationPlanServiceImpl implements IRotationPlanService {
    @Resource
    private RotationPlanMapper adRotationPlanMapper;

    @Resource
    private RedisCache redisCache;

    @Resource
    private CreateCarouselPlanUtil25 createCarouselPlanUtil25;

    @Resource
    private ISysParamService configService;

    @Resource
    private ScreenMaterialMapper screenMaterialMapper;

    @Resource
    private PlanSettingMapper planSettingMapper;

    /**
     * 查询广告轮播计划 精确到列车
     *
     * @param carouselPlanParamVo
     * @return 广告轮播计划
     */
    @Override
    public AjaxResult selectPlanByParamVo(CarouselPlanParamReq25 carouselPlanParamVo) {
        CarouselPlan25 carouselPlan = adRotationPlanMapper.selectPlanByParamVo(carouselPlanParamVo);
        List<CarouselPlanPicDto25> list = new ArrayList<>();
        if (carouselPlan != null) {
            list = JSON.parseArray(carouselPlan.getCarouselPlan(), CarouselPlanPicDto25.class);
        } else {
            PlanSetting25 planSetting = planSettingMapper.selectPlanSettingById(carouselPlanParamVo.getGroupId());
            List<CarouselPlanDayPicsDto25> dataList = getDayCarouselPlanData(planSetting).getDataList();
            for (CarouselPlanDayPicsDto25 item: dataList) {
                list.add(new CarouselPlanPicDto25()
                        .setId(item.getAdMaterialUrlDto25().getId())
                        .setHour(item.getHour()).setTime(item.getTime())
                        .setUrl(item.getAdMaterialUrlDto25().getOriginalPicUrl()));
            }
        }
        // 封装返回的数据
        LinkedHashMap<String, List<CarouselPlanPicDto25>> mapData =
                list.stream().collect(Collectors.groupingBy(CarouselPlanPicDto25::getHour,LinkedHashMap::new,Collectors.toCollection(ArrayList::new)));
        return AjaxResult.success(mapData);
    }

    @Override
    public AjaxResult pushRedis(){
        createCarouselPlanUtil25.createScreenAdvertiseTimeData();
        return AjaxResult.success();
    }

    /**
     * 说明：获取整日的图片轮播数据
     */
    @Override
    public CarouselPlanDayDto25 getDayCarouselPlanData(PlanSetting25 setting){
        PlanTaskDto dto = getCommonParam();
        List<MaterialUrlDto25> picList = new ArrayList<>();
        if (setting.getMaterialIds() != null && setting.getMaterialIds().length()>0){
            picList = screenMaterialMapper.selectMaterialByIds(Convert.toLongArray(setting.getMaterialIds().split(",")));
        } else {
            picList = dto.getPublicMaterials();
        }

        // 计算运营开始和结束时间
        Date startDate = Convert.toDate(dto.getStartTime());
        Date endDate = Convert.toDate(dto.getEndTime());
        // 分钟数
        long betweenMinute = DateUtil.between(startDate, endDate, DateUnit.MINUTE);
        // 生成图片列表
        int timeCount = (int) betweenMinute / setting.getCircleTime();
        List<MaterialUrlDto25> minuteMaterials = new ArrayList<>();
        for (int i = 0; i < timeCount; i++) {
            minuteMaterials.add(picList.get(i%picList.size()));
        }
        // 处理图片轮询时间
        LinkedList<CarouselPlanDayPicsDto25> list = new LinkedList<>();
        getTimeMapData(startDate, list, betweenMinute, minuteMaterials, 0, setting.getCircleTime());

        // 不论有没有图片，都要把公益图片放进去---公益图必须包含 id=1 的图片
        Long[] ids = Convert.toLongArray(picList.stream().map(item->item.getId()).collect(Collectors.toList()));
        if (!ArrayUtil.contains(ids, SubwayConstant25.AD_PUBLIC_BENEFIT_MATERIAL)){
            picList.addAll(dto.getPublicMaterials());
        }

        CarouselPlanDayDto25 result = new CarouselPlanDayDto25();
        result.setStartDate(dto.getStartTime())
                .setEndDate(dto.getEndTime())
                .setDataList(list)
                .setImageList(picList)
                .setDisplayTime(setting.getCircleTime())
                .setHbiTime(dto.getMucHBI());
       return result;
    }

    private PlanTaskDto getCommonParam(){
        PlanTaskDto planTaskDto = new PlanTaskDto();
        // 设备运行开始时间
        String start = configService.selectConfigByKey("device25.run.start");
        planTaskDto.setStartTime(StringUtils.isNotEmpty(start)? start: SubwayConstant25.STATION_START_TIME);
        // 设备运行结束时间
        String end = configService.selectConfigByKey("device25.run.end");
        planTaskDto.setEndTime(StringUtils.isNotEmpty(end)? end: SubwayConstant25.STATION_END_TIME);
        // 心跳周期
        String hbi = configService.selectConfigByKey("device25.muc.hbi");
        planTaskDto.setMucHBI(StringUtils.isNotEmpty(hbi)? Integer.valueOf(hbi): SubwayConstant25.MUC_HBI);
        // 公益图片
        planTaskDto.setPublicMaterials(screenMaterialMapper.selectMaterialListForPublic());
        return planTaskDto;
    }

    /**
     * 根据时间获取封装可展示的轮播数据
     *
     * @param startDate     时间段的开始时间
     * @param list       轮播数据
     * @param betweenMinute 时间差（分钟）
     * @param materials     图片素材集合
     * @param index         图片素材集合的下标
     * @param displayTime   图片轮播周期
     * @return
     */
    private int getTimeMapData(Date startDate, LinkedList<CarouselPlanDayPicsDto25> list, long betweenMinute, List<MaterialUrlDto25> materials, int index, int displayTime) {
        Date start = new Date(startDate.getTime());
        for (int i = 0; i < betweenMinute; i += displayTime, index++) {
            CarouselPlanDayPicsDto25 plan = new CarouselPlanDayPicsDto25();
            Date end = new Date(start.getTime() + displayTime * 60 * 1000);
            plan.setHour(DateUtil.format(start, "HH") + ":00");
            plan.setTime(DateUtil.format(start, "HH:mm") + "~" + DateUtil.format(end, "HH:mm"));
            plan.setAdMaterialUrlDto25(materials.get(index));
            list.add(plan);
            start = end;
        }
        return index;
    }

    @Override
    public CarouselPlanRedisDto25 getCarouselPlanCache(String key) {
        CarouselPlanRedisDto25 dto25 = redisCache.getCacheObject(SubwayConstant25.CAROUSEL_PLAN_KEY + key);
        return dto25;
    }
}
