package com.ruoyi.advertiser25.service.impl;

import com.ruoyi.advertiser25.domain.AdHandle25;
import com.ruoyi.advertiser25.mapper.ScreenDeviceMapper;
import com.ruoyi.advertiser25.service.IScreenDeviceService;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 把手 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@Service
public class ScreenDeviceServiceImpl implements IScreenDeviceService {
    private static final Logger log = LoggerFactory.getLogger(ScreenDeviceServiceImpl.class);

    @Resource
    private ScreenDeviceMapper screenDeviceMapper;

    /**
     * 查询把手
     *
     * @param id 把手 ID
     * @return 把手
     */
    @Override
    public AdHandle25 selectAdHandleById(Long id)
    {
        return screenDeviceMapper.selectAdHandleById(id);
    }

    @Override
    public AdHandle25 selectAdHandleByImei(String imei) {
        return screenDeviceMapper.selectAdHandleByImei(imei);
    }

    /**
     * 查询把手 列表
     *
     * @param adHandle25 把手
     * @return 把手
     */
    @Override
    public List<AdHandle25> selectAdHandleList(AdHandle25 adHandle25)
    {
        List<AdHandle25> list = screenDeviceMapper.selectAdHandleList(adHandle25);
        return list;
    }

    /**
     * 新增把手
     *
     * @param adHandle25 把手
     * @return 结果
     */
    @Override
    public int insertAdHandle(AdHandle25 adHandle25)
    {
        adHandle25.setCreateBy(SecurityUtils.getUsername());
        adHandle25.setCreateTime(DateUtils.getNowDate());
        return screenDeviceMapper.insertAdHandle(adHandle25);
    }

    /**
     * 修改把手
     *
     * @param adHandle25 把手
     * @return 结果
     */
    @Override
    public int updateAdHandle(AdHandle25 adHandle25)
    {
        adHandle25.setUpdateBy(SecurityUtils.getUsername());
        adHandle25.setUpdateTime(DateUtils.getNowDate());
        return screenDeviceMapper.updateAdHandle(adHandle25);
    }

    /**
     * 批量删除把手
     *
     * @param ids 需要删除的把手 ID
     * @return 结果
     */
    @Override
    public int deleteAdHandleByIds(Long[] ids)
    {
        return screenDeviceMapper.deleteAdHandleByIds(ids);
    }

    /**
     * 删除把手 信息
     *
     * @param id 把手 ID
     * @return 结果
     */
    @Override
    public int deleteAdHandleById(Long id)
    {
        return screenDeviceMapper.deleteAdHandleById(id);
    }

    /**
     * 验证编号是否重复
     */
    @Override
    public String checkNumUnique(AdHandle25 adHandle25) {
        int count = screenDeviceMapper.checkNumUnique(adHandle25);
        return count > 0 ? UserConstants.NOT_UNIQUE : UserConstants.UNIQUE;
    }

    /**
     * 根据心跳修改拉手信息
     */
    @Override
    public void updateHandleByHeartbeat(AdHandle25 adHandle25) {
        screenDeviceMapper.updateHandleByHeartbeat(adHandle25);
    }

    /**
     * 修改seId
     */
    @Override
    public void updateSeId(String imei, String seId) {
        screenDeviceMapper.updateSeId(imei, seId);
    }

    /**
     * 更新关联关系
     */
    public int updateAdHandleRelation(AdHandle25 adHandle25) {
        return screenDeviceMapper.updateAdHandleRelation(adHandle25);
    }
}
