package com.ruoyi.advertiser25.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.core.util.IdUtil;
import com.ruoyi.advertiser25.constant.McuSignatureConstant25;
import com.ruoyi.advertiser25.constant.SeSignature25;
import com.ruoyi.advertiser25.domain.Material25;
import com.ruoyi.advertiser25.enums.MaterialStatusEnum25;
import com.ruoyi.advertiser25.mapper.ScreenMaterialMapper;
import com.ruoyi.advertiser25.service.IScreenMaterialService;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.setsm25.service.SeActivating25Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.HttpClientErrorException;

import javax.annotation.Resource;
import javax.imageio.stream.FileImageOutputStream;
import java.io.File;
import java.util.List;

/**
 * 广告素材 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-13
 */
@Service
@Slf4j
public class ScreenMaterialServiceImpl implements IScreenMaterialService {
    
    @Resource
    private ScreenMaterialMapper screenMaterialMapper;

    @Resource
    private SeActivating25Service activatingService;

    @Override
    public List<Material25> selectMaterialList(Material25 material25) {
        List<Material25> material25List = screenMaterialMapper.selectMaterialList(material25);
        for (Material25 pic : material25List) {
            pic.setStatusDesc(MaterialStatusEnum25.findDescByCode(pic.getStatus()));
        }
        return material25List;
    }

    @Override
    public int insertMaterial(Material25 material25) {
        material25.setStatus(MaterialStatusEnum25.WAIT_SUBMIT.getCode());
        material25.setCreateBy(SecurityUtils.getUsername());
        material25.setCreateTime(DateUtils.getNowDate());
        int result = screenMaterialMapper.insertMaterial(material25);
        // 65525是演示模式的图片，需要跳过
        if (material25.getId() == 65525L){
            material25.setId(null);
            screenMaterialMapper.insertMaterial(material25);
            screenMaterialMapper.deleteMaterialByIds(new Long[]{65525L});
        }
        return result;
    }

    @Override
    public Material25 selectMaterialById(Long id) {
        return screenMaterialMapper.selectMaterialById(id);
    }

    @Override
    public int updateMaterial(Material25 material25) {
        material25.setUpdateBy(SecurityUtils.getUsername());
        material25.setUpdateTime(DateUtils.getNowDate());
        return screenMaterialMapper.updateMaterial(material25);
    }

    @Override
    public int deleteMaterialByIds(Long[] ids) {
        return screenMaterialMapper.deleteMaterialByIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult passMaterial(Material25 picId) {
        Material25 material25 = screenMaterialMapper.selectMaterialById(picId.getId());
        if (material25 == null || "".equals(material25.getConvertedFileUrl())){
            return AjaxResult.error("签名失败！未查询到图片信息。");
        }

        String tscUrl = RuoYiConfig.getProfile()+ material25.getConvertedFileUrl().substring(material25.getConvertedFileUrl().indexOf("/upload"));
        File file;
        FileReader fileReader;
        try {
            file = new File(tscUrl);
            fileReader = new FileReader(file);
        }catch (IORuntimeException e){
            return AjaxResult.error("签名失败！未读取到文件，请重新上传文件。");
        }
        byte[] infoImage = fileReader.readBytes();
        byte[] temp = new byte[infoImage[McuSignatureConstant25.SIGNATURE_LENGTH_INDEX]];
        System.arraycopy(infoImage, McuSignatureConstant25.SIGNATURE_DATA_INDEX, temp, 0, temp.length);

//        //应用方流水号  @TODO 先不管签名
//        String requestId = IdUtil.simpleUUID();
//        String time = DateUtil.format(DateUtil.date(), "yyyyMMddHHmmss");
//        SeSignature25 seSignature25 = new SeSignature25();
//        seSignature25.setApTxnNo(requestId).setSignType("01").setDataForSign(Convert.toHex(temp)).setTimes(time);
//        String signature;
//        try {
//            signature = activatingService.signature(requestId, seSignature25);
//        }catch (HttpClientErrorException.BadRequest e){
//            log.error(e.getMessage());
//            return AjaxResult.error("签名失败！复旦微签名接口连接错误，请联系管理员。");
//        }catch (NullPointerException e){
//            log.error(e.getMessage());
//            return AjaxResult.error("签名失败！复旦微签名接口返回信息为空，请联系管理员。");
//        }catch (ServiceException e){
//            log.error(e.getMessage());
//            return AjaxResult.error("签名失败！复旦微签名接口出现其他未知异常，请联系管理员。");
//        }
//        if ("".equals(seSignature25)){
//            return AjaxResult.error("签名失败！复旦微签名接口返回信息为空，请联系管理员。");
//        }
//
//        byte[] signResult = Convert.hexToBytes(signature);
//        System.arraycopy(signResult, 0, infoImage, McuSignatureConstant25.SIGNATURE_DATA_INDEX, signResult.length);
//        infoImage[McuSignatureConstant25.IS_SIGNATURE_INDEX] = McuSignatureConstant25.YES_SIGNATURE;
//        infoImage[McuSignatureConstant25.SIGNATURE_LENGTH_INDEX] = (byte) signResult.length;
//
//        material25.setConvertedFileUrl(material25.getConvertedFileUrl().replace(material25.getConvertedFileUrl()
//                .substring(material25.getConvertedFileUrl().lastIndexOf("/") +1), material25.getId() + ".LSC"));
//        try {
//            FileImageOutputStream imageOutput = new FileImageOutputStream(file);
//            imageOutput.write(infoImage, 0, infoImage.length);
//            imageOutput.close();
//            file.renameTo(new File(RuoYiConfig.getProfile()+ material25.getConvertedFileUrl().substring(material25.getConvertedFileUrl().indexOf("/upload"))));
//        }catch (Exception e){
//            log.error(e.getMessage());
//            return AjaxResult.error("签名失败！保存签名文件出错。");
//        }
        Material25 updatedMaterial25 = new Material25();
        updatedMaterial25.setId(picId.getId());
        updatedMaterial25.setConvertedFileUrl(material25.getConvertedFileUrl());
        updatedMaterial25.setStatus(MaterialStatusEnum25.PASS.getCode());
        // 新增图片签名时间
        byte[] signTime = new byte[4];
        System.arraycopy(infoImage, McuSignatureConstant25.FILE_CREATE_TIME_START, signTime, 0, McuSignatureConstant25.FILE_CREATE_TIME_LENGTH);
        updatedMaterial25.setSignTime(Convert.toHex(signTime));
        screenMaterialMapper.updateMaterial(updatedMaterial25);
        return AjaxResult.success("签名成功！");
    }

    @Override
    public AjaxResult demo(Long id){
        screenMaterialMapper.updateDemoPic();
        Material25 pic = screenMaterialMapper.selectMaterialById(id);
        pic.setCreateBy(SecurityUtils.getUsername());
        pic.setCreateTime(DateUtils.getNowDate());
        screenMaterialMapper.insertDemoPic(pic);
        return AjaxResult.success();
    }

    @Override
    public AjaxResult getDemoPic(){
        return AjaxResult.success(screenMaterialMapper.selectDemoPic());
    }
}
