package com.ruoyi.advertiser25.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.core.io.file.FileWriter;
import cn.hutool.core.util.IdUtil;
import com.ruoyi.advertiser25.constant.McuSignatureConstant25;
import com.ruoyi.advertiser25.constant.SeSignature25;
import com.ruoyi.advertiser25.domain.Upgrade25;
import com.ruoyi.advertiser25.mapper.UpgradeMapper;
import com.ruoyi.advertiser25.service.IUpgradeService;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.setsm25.service.SeActivating25Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * 固件升级 Service业务层处理
 * 
 * @author caiweigan
 * @date 2022-02-07
 */
@Service
public class UpgradeServiceImpl implements IUpgradeService
{
    @Autowired
    private UpgradeMapper upgradeMapper;

    @Autowired
    private SeActivating25Service activatingService;


    /**
     * 查询固件升级 
     * 
     * @param upgradeId 固件升级 主键
     * @return 固件升级 
     */
    @Override
    public Upgrade25 selectAdFirmwareUpgradeByUpgradeId(Long upgradeId)
    {
        return upgradeMapper.selectAdFirmwareUpgradeByUpgradeId(upgradeId);
    }

    /**
     * 查询固件升级 列表
     * 
     * @param adFirmwareUpgrade 固件升级 
     * @return 固件升级 
     */
    @Override
    public List<Upgrade25> selectAdFirmwareUpgradeList(Upgrade25 adFirmwareUpgrade)
    {
        return upgradeMapper.selectAdFirmwareUpgradeList(adFirmwareUpgrade);
    }

    /**
     * 新增固件升级 
     * 
     * @param adFirmwareUpgrade 固件升级 
     * @return 结果
     */
    @Override
    public int insertAdFirmwareUpgrade(Upgrade25 adFirmwareUpgrade)
    {
//        @TODO 临时注释掉验签
//        signature(adFirmwareUpgrade);
        adFirmwareUpgrade.setCreateBy(SecurityUtils.getUsername());
        adFirmwareUpgrade.setCreateTime(DateUtils.getNowDate());
        return upgradeMapper.insertAdFirmwareUpgrade(adFirmwareUpgrade);
    }

    /**
     * 修改固件升级 
     * 
     * @param adFirmwareUpgrade 固件升级 
     * @return 结果
     */
    @Override
    public int updateAdFirmwareUpgrade(Upgrade25 adFirmwareUpgrade)
    {
        signature(adFirmwareUpgrade);
        adFirmwareUpgrade.setUpdateBy(SecurityUtils.getUsername());
        adFirmwareUpgrade.setUpdateTime(DateUtils.getNowDate());
        return upgradeMapper.updateAdFirmwareUpgrade(adFirmwareUpgrade);
    }

    /**
     * 批量删除固件升级 
     * 
     * @param upgradeIds 需要删除的固件升级 主键
     * @return 结果
     */
    @Override
    public int deleteAdFirmwareUpgradeByUpgradeIds(Long[] upgradeIds)
    {
        return upgradeMapper.deleteAdFirmwareUpgradeByUpgradeIds(upgradeIds);
    }

    /**
     * 删除固件升级 信息
     * 
     * @param upgradeId 固件升级 主键
     * @return 结果
     */
    @Override
    public int deleteAdFirmwareUpgradeByUpgradeId(Long upgradeId)
    {
        return upgradeMapper.deleteAdFirmwareUpgradeByUpgradeId(upgradeId);
    }

    /**
     * 修改发布状态
     */
    @Override
    public int updatePublishStatus(Long upgradeId) {
        return upgradeMapper.updatePublishStatus(upgradeId);
    }

    /**
     * SE-TSM签名
     */
    @Override
    public void signature(Upgrade25 adFirmwareUpgrade) {
        // 磁盘路径
        String filePath = RuoYiConfig.getProfile() + adFirmwareUpgrade.getAttachment().replace("/profile", "");
        FileReader fileReader = new FileReader(filePath);
        // 解析固件包字节
        byte[] fileDatas = fileReader.readBytes();
        byte[] signatureDatas = new byte[fileDatas[McuSignatureConstant25.SIGNATURE_LENGTH_INDEX]];
        System.arraycopy(fileDatas, McuSignatureConstant25.SIGNATURE_DATA_INDEX, signatureDatas, 0, signatureDatas.length);
        // 数字签名
        SeSignature25 seSignature = new SeSignature25();
        String requestId = IdUtil.simpleUUID();
        String time = DateUtils.dateTimeNow();
        seSignature.setApTxnNo(requestId)
                .setSignType("01")
                .setDataForSign(Convert.toHex(signatureDatas))
                .setTimes(time);
        String signature = activatingService.signature(requestId, seSignature);
        // 签名后，替换固件包指定位置字节
        byte[] signatureResultDatas = Convert.hexToBytes(signature);
        System.arraycopy(signatureResultDatas, 0, fileDatas, McuSignatureConstant25.SIGNATURE_DATA_INDEX, signatureResultDatas.length);
        fileDatas[McuSignatureConstant25.IS_SIGNATURE_INDEX] = McuSignatureConstant25.YES_SIGNATURE;
        fileDatas[McuSignatureConstant25.SIGNATURE_LENGTH_INDEX] = (byte) signatureResultDatas.length;
        FileWriter fileWriter = new FileWriter(filePath);
        fileWriter.write(fileDatas, 0, fileDatas.length);
    }

    /**
     * 根据屏幕类型、硬件版本
     * 已发布状态、软件版本号排序 查询最新的固件包
     */
    @Override
    public Upgrade25 selectLatestFirmware(Upgrade25 adFirmwareUpgrade) {
        return upgradeMapper.selectLatestFirmware(adFirmwareUpgrade);
    }

    /**
     * 上传固件包
     * @param file
     * @return
     */
    @Override
    public Upgrade25 upload(MultipartFile file) throws IOException {
        byte[] fileDatas = file.getBytes();
        String screenType = Convert.toHex(new byte[]{fileDatas[0x13], fileDatas[0x14]});
        String hardwareVersion = Convert.toHex(new byte[]{fileDatas[0x21]});
        String softwareVersion = Convert.toHex(new byte[]{fileDatas[0x22]});
        String majorVersion = softwareVersion.substring(0, 1);
        String subVersion = softwareVersion.substring(1, 2);

        // 根据屏幕类型、硬件版本、比较软件版本号
        Upgrade25 param = new Upgrade25().setScreenType(screenType).setHardwareVersion(Integer.parseInt(hardwareVersion));
        Upgrade25 latestFirmware = selectLatestFirmware(param);
        if (latestFirmware != null) {
            String latestSoftwareVersion = latestFirmware.getMajorVersion().toString() + latestFirmware.getSubVersion();
            int sV = Integer.parseInt(softwareVersion);
            int latestSV = Integer.parseInt(latestSoftwareVersion);
            if (sV == latestSV) {
                throw new RuntimeException("当前版本固件包已存在");
            } else if (sV < latestSV) {
                throw new RuntimeException("软件版本不能小于已上传的固件包");
            }
        }

        // 上传文件
        String filePath = RuoYiConfig.getUploadPath();
        String fileName = FileUploadUtils.upload(filePath, file);

        Upgrade25 upgrade = new Upgrade25()
                .setScreenType(screenType)
                .setHardwareVersion(Integer.parseInt(hardwareVersion))
                .setMajorVersion(Integer.parseInt(majorVersion))
                .setSubVersion(Integer.parseInt(subVersion))
                .setAttachment(fileName);
        return upgrade;
    }

}
