package com.ruoyi.advertiser25.task;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.ruoyi.advertiser25.constant.SubwayConstant25;
import com.ruoyi.advertiser25.domain.CarouselPlan25;
import com.ruoyi.advertiser25.domain.PlanSetting25;
import com.ruoyi.advertiser25.domain.dto.*;
import com.ruoyi.advertiser25.mapper.RotationPlanMapper;
import com.ruoyi.advertiser25.service.*;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.mcu25.config.McuSocketConfig25;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 生产轮播计划数据的定时任务
 *
 * @author zhangzhanwei
 * @date 2021-09-14
 */

@Component("carouselPlanTask25")
@Slf4j
public class CreateCarouselPlanUtil25 {

    @Resource
    private RotationPlanMapper rotationPlanMapper;

    @Resource
    private RedisCache redisCache;

    @Resource
    private IRotationPlanService planService;

    @Resource
    private IPlanSettingService planSettingService;

    @Resource
    private McuSocketConfig25 mcuSocketConfig25;

    /**
     * 创建轮播计划数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void createScreenAdvertiseTimeData() {
        log.info("开始生产25寸屏幕的轮播计划数据.................");
        Date startTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD, DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD,DateUtils.getNowDate()));
        Date endTime = startTime;
        // 生成数据时先删除计划表里的数据
        rotationPlanMapper.deleteAdCarouselPlanByTime(startTime, endTime);
        // 要生成的轮播
        createCarouselPlanDataBySubwayLine(startTime);
        log.info("生产25寸屏幕的轮播计划数据结束.................");
    }

    /**
     * @param thatDay 当前日期
     * @return CarouselPlan 轮播计划
     */
    public void createCarouselPlanDataBySubwayLine(Date thatDay) {
        List<PlanSetting25> groupList = planSettingService.selectPlanSettingList(new PlanSetting25());
        for (PlanSetting25 setting: groupList) {
            CarouselPlan25 carouselPlan = new CarouselPlan25();
            carouselPlan.setGroupId(setting.getId());
            carouselPlan.setAdvertiseDate(thatDay);
            // 获取轮播数据
            CarouselPlanDayDto25 dayDto = planService.getDayCarouselPlanData(setting);
            List<CarouselPlanPicDto25> picList = new ArrayList<>();
            dayDto.getDataList().stream().forEach(item->{
                picList.add(new CarouselPlanPicDto25()
                        .setId(item.getAdMaterialUrlDto25().getId())
                        .setHour(item.getHour()).setTime(item.getTime())
                        .setUrl(item.getAdMaterialUrlDto25().getOriginalPicUrl()));
            });
            carouselPlan.setCarouselPlan(JSON.toJSONString(picList));
            carouselPlan.setCreateBy("admin");
            carouselPlan.setCreateTime(DateUtils.getNowDate());
            rotationPlanMapper.insertAdCarouselPlan(carouselPlan);
            // 放到redis里
            setRedisPan(carouselPlan.getGroupId(), picList, dayDto);
        }
    }

    /**
     * 封装数据到redis
     *
     * @param groupId
     */
    public void setRedisPan(Long groupId, List<CarouselPlanPicDto25> picList, CarouselPlanDayDto25 dayDto) {
       // 处理图片集合
        List<ImageBindDto> imageBindList = new ArrayList<>();
        dayDto.getImageList().stream().forEach(item->{
            imageBindList.add(new ImageBindDto().setId(item.getId()).setUrl(item.getConvertedFileUrl()).setTime(item.getSignTime()));
        });

        // 存入redis
        CarouselPlanRedisDto25 adCarouselPlanRedisDto25 = new CarouselPlanRedisDto25();
        adCarouselPlanRedisDto25.setStartTime(dayDto.getStartDate())
                .setEndTime(dayDto.getEndDate())
                .setPicCarouselList(picList.stream().map(item->item.getId()).toArray(Long[]::new))
                .setPicBindingList(imageBindList)
                .setDisplayTime(dayDto.getDisplayTime())
                .setHbiTime(dayDto.getHbiTime())
                .setDownHttp(mcuSocketConfig25.getHttpServer());
        redisCache.setCacheObject(SubwayConstant25.CAROUSEL_PLAN_KEY + groupId, adCarouselPlanRedisDto25);
    }

}
