package com.ruoyi.mcu25.server;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.ruoyi.advertiser25.domain.Material25;
import com.ruoyi.advertiser25.mapper.ScreenMaterialMapper;
import com.ruoyi.advertiser25.service.IScreenMaterialService;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.file.FileException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.mcu25.constant.McuBeatIdentificationConstant25;
import com.ruoyi.mcu25.constant.McuBeatPackTypeConstant25;
import com.ruoyi.mcu25.domain.AdHeartBeatPackage25;
import com.ruoyi.mcu25.domain.HttpServerBean25;
import com.ruoyi.mcu25.domain.HttpServerRedisDto;
import com.ruoyi.mcu25.util.CRC16ModbusUtil25;
import com.ruoyi.mcu25.util.DataTransitionUtil25;
import com.ruoyi.mcu25.util.ParseUtil25;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.nio.file.FileSystemException;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

/**
 * @author: jd
 * @Description:
 * @create: 2023/5/9
 */
@Slf4j
@RestController
@RequestMapping("/httpServer")
public class HttpServer25 {

    private static final String token = "SbteA!A6lpOrmI@G.vENE-0KUPaVdrZjUiO(aA$@7jJo31b1Yzuj68bvIi+=Um@k";

    @Resource
    private DealServer25 dealServer25;

    @Resource
    private IScreenMaterialService materialService;

    @Resource
    private ScreenMaterialMapper screenMaterialMapper;

    @Resource
    private RedisCache redisCache;

    @GetMapping("/{fileName}")
    public void fileDownload(@PathVariable("fileName") String fileName, HttpServletResponse response, HttpServletRequest request) {
        String ip = IpUtils.getIpAddr(request);
        String range = request.getHeader("Range");
        log.info("开始--设备请求下载！设备ip:{} ----------------------------------------------------------", ip);
        log.info("请求时间：{}，请求参数：{}，range：{}", DateUtil.now(), fileName, range);
        try {
            // 先尝试从redis读数据--读取不到在查询数据库时加入缓存。注意：如果图片支持根据id修改图片文件地址，则需要把redis对应文件清理掉
            if (!readFromRedis(fileName, response, range)){
                // 禁止目录上跳级别
                if (StringUtils.contains(fileName, "..")) {
                    throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
                }
                String id = fileName.split("\\.")[0];
                // 演示模式固定下载 65535.GSC
                if ("65535".equals(id)){
                    id = screenMaterialMapper.selectDemoPic().get("oriId").toString();
                }
                Material25 material25 = materialService.selectMaterialById(Long.valueOf(id));
                if (material25 == null || StringUtils.isEmpty(material25.getConvertedFileUrl())){
                    throw new Exception(StringUtils.format("系统未查询到文件：({})，下载失败。 ", fileName));
                }
                // 本地资源路径
                String localPath = RuoYiConfig.getProfile();
                // 数据库资源地址
                String downloadPath = localPath + StringUtils.substringAfter(material25.getConvertedFileUrl(), Constants.RESOURCE_PREFIX);
                // 下载名称
                String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
                FileUtils.setAttachmentResponseHeader(response, downloadName);
                writeBytes(fileName, downloadPath, response, request);
            }
        } catch (Exception e) {
            log.error("下载文件失败", e);
            log.error("结束--下载失败！设备ip:{}----------------------------------------------------------", ip);
            return;
        }

        log.info("结束--下载完成！设备ip:{}----------------------------------------------------------", ip);
    }

    private boolean readFromRedis( String fileName, HttpServletResponse response, String range) throws IOException {
        String id = fileName.split("\\.")[0];
        HttpServerRedisDto redisDto = redisCache.getCacheObject("PicServer:id_" + id + ":range" + range);
        if (redisDto != null){
            OutputStream os = null;
            try {
                FileUtils.setAttachmentResponseHeader(response, id);
                response.setHeader("Access-Control-Allow-Origin","*");
                response.setHeader("Accept-Ranges", "bytes");
                if (StringUtils.isNotEmpty(range)){
                    response.setStatus(HttpServletResponse.SC_PARTIAL_CONTENT);
                    response.setHeader("Content-Range", redisDto.getContentRange());
                }
                response.setContentLength(redisDto.getContentLength());

                os = response.getOutputStream();
                for (int i = 0; i < redisDto.getData().size(); i++) {
                    os.write(redisDto.getData().get(i), 0, redisDto.getData().get(i).length);
                }
            }catch (Exception e){
                throw e;
            }finally {
                os.close();
            }
            return true;
        }
        return false;
    }

    /**
     * 说明：需要支持断点续传
     */
    public void writeBytes(String filename, String filePath, HttpServletResponse response, HttpServletRequest request) throws IOException {
        String range = request.getHeader("Range");
        if (StringUtils.isNotEmpty(range) && range.contains(",")){
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            response.setContentLength(0);
            response.setHeader("Error-Info", "The System Does Not Support Multi Range");
            throw new FileSystemException("系统不支持多段range");
        }
        response.setHeader("Access-Control-Allow-Origin","*");
        response.setHeader("Accept-Ranges", "bytes");

        HttpServerRedisDto redisDto = new HttpServerRedisDto();
        redisDto.setId(filename.split("\\.")[0]);
        OutputStream os = response.getOutputStream();
        RandomAccessFile rf = null;
        try {
            File file = new File(filePath);
            if (!file.exists()) {
                throw new FileNotFoundException(filePath);
            }
            long fileMaxSize = file.length() - 1;
            redisDto.setMaxSize(fileMaxSize);
            rf = new RandomAccessFile(file, "r");
            int downLength = 0;
            // 是否断点续传
            if (StringUtils.isNotEmpty(range)){
                // 续传返回206
                response.setStatus(HttpServletResponse.SC_PARTIAL_CONTENT);
                // 读取范围
                String contentRange = range.replace("=", " ") + "/" + fileMaxSize;
                redisDto.setContentRange(contentRange);
                response.setHeader("Content-Range", contentRange);
                String[] readRange = range.replace("bytes=","").trim().split("-");
                Long start = 0L;
                Long end = 0L;
                if (readRange.length == 1){
                    start = Convert.toLong(readRange[0]);
                    end = fileMaxSize;
                } else {
                    start = StringUtils.isNotEmpty(readRange[0])? Convert.toLong(readRange[0]) : start;
                    end = Convert.toLong(readRange[1]);
                    end = end>fileMaxSize?fileMaxSize:end;
                }
                // 跳过不需要下载的
                if (start != 0){
                    rf.seek(start);
                }
                downLength = (int) (end - start) + 1;
                // If-None-Match-ETag：相同则返回304，让客户端使用本地缓存--优化
            }
            if (downLength == 0){
                downLength = (int) fileMaxSize + 1;
            }
            redisDto.setContentLength(downLength);
            response.setContentLength(downLength);
            List<byte[]> list = new ArrayList<>();
            byte[] b = new byte[1024];
            int length;
            while (downLength > 0 && (length = rf.read(b)) > 0) {
                list.add(Arrays.copyOf(b, length>downLength? downLength :length));
                os.write(b, 0, length>downLength? downLength :length);
                downLength -= length;
            }
            redisDto.setData(list);
            // 一般认为24小时内图片肯定可以下载完
            redisCache.setCacheObject("PicServer:id_" + redisDto.getId() + ":range" + range, redisDto, 24, TimeUnit.HOURS);
        } catch (IOException e) {
            throw e;
        } finally {
            IOUtils.close(os);
            rf.close();
        }
    }


    /**
     * 说明：已经不用
     */
    @PostMapping("/download")
    public String download(@RequestBody HttpServerBean25 data, HttpServletRequest req){
        String ip = IpUtils.getIpAddr(req);
        log.info("开始--接收到设备心跳！设备ip:{}\r\n ----------------------------------------------------------", ip);

        if (!token.equals(req.getHeader("token"))){
            log.error("非法访问！！！");
            return "非法访问！";
        }
        log.info("接收时间：" + DateUtil.now() + "<br/>");
        byte[] content = Base64.getDecoder().decode(data.getData());
        String hexContent = Convert.toHex(content).toUpperCase(Locale.ROOT);
        log.info("接收数据：长度：{}\r\n HEX内容：{}\r\n BASE64内容：{}",hexContent.length(),hexContent,data.getData());

        AdHeartBeatPackage25 beatPack = ParseUtil25.getHeartBeatPackage(content);
        if (beatPack == null) {
            log.info("心跳包内容数据为空！");
            return "心跳包内容数据为空！";
        }
        if ((beatPack.getIdentification() & 0x01) == 0x01) {
            String hex = Convert.toHex(DataTransitionUtil25.intsToBytes(beatPack.getPacketData()));
            log.info("数据SM4解密后：{}", hex.toUpperCase());
        }

        //获取校验结果
        boolean checkResult = CRC16ModbusUtil25.getCheckResult(beatPack.getPacketData(), beatPack.getCrc16());
        // crc16校验
        if (!checkResult) {
            log.error("CRC16校验失败！");
            return "CRC16校验失败！";
        }
        log.info("心跳包数据校验成功！");

        String imei = beatPack.getImeiStr();
        log.info("读取到的设备IMEI信息：**********" + imei + "*************<br/>");

        // 如果结果不是0x80则不需要回复心跳数据
        if ((beatPack.getIdentification() & McuBeatIdentificationConstant25.BEAT_IDENTIFICATION_0X80) != McuBeatIdentificationConstant25.BEAT_IDENTIFICATION_0X80) {
            log.info("非 ACK 标识，无需应答。");
            return "非 ACK 标识！";
        }
        byte[] responseBytes = null;
        if (beatPack.getPackageType() == McuBeatPackTypeConstant25.IMAGE_TRANSMIT_PACKAGE_TYPE) {
            // 图片下载&&图片文件传输状态
            responseBytes = dealServer25.imageReply(beatPack);
        }
        //回复消息
        if (responseBytes != null) {
            log.info("结束--数据发送完成！设备ip:{}\r\n----------------------------------------------------------", ip);
            ByteBuffer writebuffer = ByteBuffer.wrap(responseBytes);
            return  writebuffer.toString();
        }
        log.error("结束--数据发送失败！设备ip:{}\r\n----------------------------------------------------------", ip);
        return "服务器内部异常，未获取到资源信息！";
    }
}
