package com.ruoyi.setsm25.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.advertiser25.constant.SeSignature25;
import com.ruoyi.setsm25.config.SE_TSMKeyConfig25;
import com.ruoyi.setsm25.domain.ActivatingCommand25;
import com.ruoyi.setsm25.domain.SeActivating25;
import com.ruoyi.setsm25.domain.SeResult25;
import com.ruoyi.setsm25.domain.SeSignatureResult25;
import com.ruoyi.setsm25.exception.McuServiceException25;
import com.ruoyi.setsm25.service.SeActivating25Service;
import com.ruoyi.setsm25.util.SignatureUtil25;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

/**
 * @author: jd
 * @Description:
 * @create: 2022/12/7
 */
@Slf4j
@Service
public class SeActivating25ServiceImpl implements SeActivating25Service {

    @Resource
    private SE_TSMKeyConfig25 se_tsmKeyConfig25;

    @Resource(name = "restTemplateForAuthentication25")
    private RestTemplate restTemplateForAuthentication;

    /**
     * 激活业务接口
     *
     * @param requestId    请求标识，作为判断重发报文的唯一标志
     * @param seActivating25 Activating – 激活业务码数据
     */
    @Override
    public String activating(String requestId, SeActivating25 seActivating25) {

        //se-tsm公钥
        String sePubKey = se_tsmKeyConfig25.getKey().getSePubKey();
        String pubKey = se_tsmKeyConfig25.getKey().getPubKey();
        //平台私钥
        String priKey = se_tsmKeyConfig25.getKey().getPriKey();

        StringBuffer url = new StringBuffer();
        url.append(se_tsmKeyConfig25.getSeTesm().getProtocol());
        url.append(se_tsmKeyConfig25.getSeTesm().getHostAndPort());
        url.append(se_tsmKeyConfig25.getSeTesm().getActivatingUrl());
        JSONObject activating = JSONObject.parseObject(JSONObject.toJSONString(seActivating25));
        log.debug("activating", activating);
        String bizData = Base64.encode(activating.toString());

        //签名数据
        byte[] signature = SignatureUtil25.sign(bizData.getBytes(), priKey);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("request-id", requestId);
        JSONObject data = new JSONObject();
        data.put("bizData", bizData);
        data.put("signature", signature);
        JSONObject bizSignedData = new JSONObject();
        bizSignedData.put("bizSignedData", data);

        log.debug("bizSignedData", bizSignedData);
        HttpEntity<JSONObject> httpEntity = new HttpEntity<JSONObject>(bizSignedData, headers);

        Object activatingCommands = null;//restTemplateUtil.sendPost(url.toString(), httpEntity);

        log.debug("activatingCommands", activatingCommands);

        SeResult25 seResult25 = JSONUtil.toBean(activatingCommands.toString(), SeResult25.class);
        log.debug("seResult", seResult25);
        if(null== seResult25.getBizSignedData25()){
            log.error("seResult:", seResult25);
            throw new McuServiceException25("激活失败，接收到的bizSignedData为空");
        }

        StringBuffer activatingData = new StringBuffer();//se激活数据
        //验签
        boolean verify = Optional.ofNullable(SignatureUtil25.verify(seResult25.getBizSignedData25().getBizData().getBytes(), Base64.decode(seResult25.getBizSignedData25().getSignature()), sePubKey)).orElse(false);
        if (verify) {
            String decodeStr = Base64.decodeStr(seResult25.getBizSignedData25().getBizData());
            log.debug(decodeStr);
            JSON parse = JSONUtil.parse(decodeStr);
            Object activatingCommands1 = parse.getByPath("activatingCommands");
            String aa = JSONObject.toJSONString(activatingCommands1);
            List<ActivatingCommand25> activatingCommands225 = null;//JSONArray.parseArray(aa, ActivatingCommand.class);

            activatingCommands225.forEach(activatingCommand25 -> {
                log.debug("activatingCommand", activatingCommand25);
                activatingData.append(activatingCommand25.getApdu());
            });
        } else {
            throw new McuServiceException25("验签失败!");
        }

        return activatingData.toString();
    }

    @Override
    public String signature(String requestId, SeSignature25 seSignature25) {
        // se-tsm公钥
        String sePubKey = se_tsmKeyConfig25.getKey().getSePubKey();
        // 平台私钥
        String priKey = se_tsmKeyConfig25.getKey().getPriKey();

        // 签名数据
        JSONObject activating = JSONObject.parseObject(JSONObject.toJSONString(seSignature25));
        String bizData = Base64.encode(activating.toString());
        byte[] signature = SignatureUtil25.sign(bizData.getBytes(), priKey);

        // 请求参数
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("request-id", requestId);
        JSONObject data = new JSONObject();
        data.put("bizData", bizData);
        data.put("signature", signature);
        JSONObject bizSignedData = new JSONObject();
        bizSignedData.put("bizSignedData", data);
        StringBuffer url = new StringBuffer();
        url.append(se_tsmKeyConfig25.getSeTesm().getProtocol());
        url.append(se_tsmKeyConfig25.getSeTesm().getHostAndPort());
        url.append(se_tsmKeyConfig25.getSeTesm().getSignatureUrl());
        HttpEntity<JSONObject> httpEntity = new HttpEntity<JSONObject>(bizSignedData, headers);
        SeResult25 seResult25 = restTemplateForAuthentication.postForObject(url.toString(), httpEntity, SeResult25.class);
        log.debug("activatingCommands", seResult25);

        //验签
        boolean verify = SignatureUtil25.verify(seResult25.getBizSignedData25().getBizData().getBytes(), Base64.decode(seResult25.getBizSignedData25().getSignature()), sePubKey);
        if (verify) {
            String decodeStr = Base64.decodeStr(seResult25.getBizSignedData25().getBizData());
            SeSignatureResult25 seSignatureResult25 = JSONUtil.toBean(decodeStr, SeSignatureResult25.class);
            log.debug("seSignatureResult", seSignatureResult25);
            return seSignatureResult25.getSignResult();
        } else {
            throw new McuServiceException25("验签失败!");
        }
    }
}
